/*
 * This program writes a valid value to IA32_PAT MSR but at CPL != 0.
 *
 * This should result in a #GP fault and is generated by the processor
 * directly rather than the hypervisor. In response to this a SIGBUS
 * signal is delivered to the process and core dump generated. The core
 * file can be examined using gdb to ensure that the faulting instruction
 * is 'wrmsr'.
 */

#include <sys/types.h>
#include <sys/sysctl.h>

#include <machine/cpufunc.h>
#include <x86/specialreg.h>

#include <stdio.h>
#include <assert.h>

int
main(void)
{
	uint64_t pat;
	int newval, error;

	newval = 1;
	error = sysctlbyname("machdep.uprintf_signal", NULL, NULL,
	    &newval, sizeof(newval));
	assert(error == 0);

        /* Initialize default PAT entries. */
        pat = PAT_VALUE(0, PAT_WRITE_BACK) |
            PAT_VALUE(1, PAT_WRITE_THROUGH) |
            PAT_VALUE(2, PAT_UNCACHED) |
            PAT_VALUE(3, PAT_UNCACHEABLE) |
            PAT_VALUE(4, PAT_WRITE_BACK) |
            PAT_VALUE(5, PAT_WRITE_THROUGH) |
            PAT_VALUE(6, PAT_UNCACHED) |
            PAT_VALUE(7, PAT_UNCACHEABLE);

	wrmsr(MSR_PAT, pat);
}
